/*-
 * Copyright (C)2017..2020 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2017..2020 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: db.c,v 1.31 2021/02/12 23:43:53 babolo Exp $"

#define BLIN_COMPAT      3
#define Bpars_COMPAT     3
#define MIFE_COMPAT      5
#define PGOBLIN_COMPAT   2
#define PGOBLIN_INTERNAL 1

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <errno.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <mife.h>
#include "pgoblin.h"

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_shurecon(pgoblin_conn *db) {                             /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = EX_OK;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->shurecon) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->shurecon(db);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_query(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) { /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    /*    */
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %s", qry);
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->query) {
        if  (!!rsv) *rsv = NULL;
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->query(db, rsv, qry);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/************************************************************************
 ************************************************************************
 **                                                                    **/
pgoblin_db_prepare(pgoblin_conn *db, pgoblin_mode **rsv, char *qry) { /**
 **                                                                    **
 ************************************************************************
 ************************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %s", qry);
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->prepare) {
        if  (!!rsv) *rsv = NULL;
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->prepare(db, rsv, qry);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/************************************************************************************************
 ************************************************************************************************
 **                                                                                            **/
pgoblin_db_execute(pgoblin_conn *db, pgoblin_mode **rsv, const void *qp, pgoblin_mode *qin) { /**
 **                                                                                            **
 ************************************************************************************************
 ************************************************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->execute) {
        if  (!!rsv) *rsv = NULL;
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->execute(db, rsv, qp, qin);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_resinfo(int kind, pgoblin_mode *rsv) {                   /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((rsv ? (BLIN_flag)*rsv : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d", kind);
    if  (!rsv) {
        errno = EFAULT;
        ex = -EX_UNAVAILABLE;
    } else if (!DBASE(*rsv) || !DBASE(*rsv)->resinfo) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(*rsv)->resinfo(kind, rsv);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

const char *
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_subinfo(int kind, pgoblin_mode *rsv, int sub) {          /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    const char *c = NULL;

#   define blin_internal_flags ((rsv ? (BLIN_flag)*rsv : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d %d", kind, sub);
    if  (!rsv) {
        errno = EFAULT;
    } else if (!DBASE(*rsv) || !DBASE(*rsv)->subinfo) {
        errno = ENOSYS;
    } else {
        c = DBASE(*rsv)->subinfo(kind, rsv, sub);
    }
    ifBLIN_QX3("- %s", c);
    return(c);
#   undef blin_internal_flags
}

char *
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_erinfo(pgoblin_mode *rsv) {                              /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    char *c = NULL;

#   define blin_internal_flags ((rsv ? (BLIN_flag)*rsv : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!rsv) {
        errno = EFAULT;
    } else if (!DBASE(*rsv) || !DBASE(*rsv)->erinfo) {
        errno = ENOSYS;
    } else {
        c = DBASE(*rsv)->erinfo(rsv);
    }
    ifBLIN_QX3("- %s", c);
    return(c);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_valinfo(int kind, pgoblin_mode *rsv, int row, int col) { /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((rsv ? (BLIN_flag)*rsv : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d %d %d", kind, row, col);
    if  (!rsv) {
        errno = EFAULT;
        ex = -EX_UNAVAILABLE;
    } else if (!DBASE(*rsv) || !DBASE(*rsv)->valinfo) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(*rsv)->valinfo(kind, rsv, row, col);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

char *
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_getvalue(pgoblin_mode *rsv, int row, int col) {          /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    char *c = NULL;

#   define blin_internal_flags ((rsv ? (BLIN_flag)*rsv : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d %d", row, col);
    if  (!rsv) {
        errno = EFAULT;
    } else if (!DBASE(*rsv) || !DBASE(*rsv)->getvalue) {
        errno = ENOSYS;
    } else {
        c = DBASE(*rsv)->getvalue(rsv, row, col);
    }
    ifBLIN_QX3("- %s", c);
    return(c);
#   undef blin_internal_flags
}

int
/**************************************************************************
 **************************************************************************
 **                                                                      **/
pgoblin_db_getstream(pgoblin_conn *db, pgoblin_mode **rsv, int quant) { /**
 **                                                                      **
 **************************************************************************
 **************************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d", quant);
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->getstream) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->getstream(db, rsv, quant);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_getcopy(pgoblin_conn *db, void **buf) {                  /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->getcopy) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->getcopy(db, buf);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_putcopy(pgoblin_conn *db, const void *buf, int len) {    /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d", len);
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->putcopy) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->putcopy(db, buf, len);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_endstream(pgoblin_conn *db) {                            /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->endstream) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->endstream(db);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_tarry(pgoblin_conn *db, int timeout) {                   /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+ %d", timeout);
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->tarry) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(db->flags)->tarry(db, timeout);
    }
    ifBLIN_QX3("- %d", ex);
    return(ex);
#   undef blin_internal_flags
}

char *
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_notifies(pgoblin_conn *db) {                             /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    char *c = NULL;

#   define blin_internal_flags ((db ? db->flags : pgoblin_default) & BLIN_MASK)
    ifBLIN_QX3("+");
    if  (!db || !DBASE(db->flags) || !DBASE(db->flags)->tarry) {
        errno = ENOSYS;
    } else {
        c = DBASE(db->flags)->notifies(db);
    }
    ifBLIN_QX3("- %s", c);
    return(c);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_finish(u_int32_t flags, int kind, void **entity) {       /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags flags
    if  (!DBASE(flags) || !DBASE(flags)->finish) {
        errno = ENOSYS;
        ex = -EX_UNAVAILABLE;
    } else {
        ex = DBASE(flags)->finish(flags, kind, entity);
    }
    return(ex);
#   undef blin_internal_flags
}

int
/**********************************************************************
 **********************************************************************
 **                                                                  **/
pgoblin_db_clear(pgoblin_mode **rsv) {                              /**
 **                                                                  **
 **********************************************************************
 **********************************************************************/
    int ex = 0;

#   define blin_internal_flags (((rsv && *rsv) ? (BLIN_flag)**rsv : pgoblin_default) & BLIN_MASK)
    if  (rsv && *rsv) {
        if  (!DBASE(**rsv) || !DBASE(**rsv)->clear) {
            errno = ENOSYS;
            ex = -EX_UNAVAILABLE;
        } else {
            ex = DBASE(**rsv)->clear(rsv);
    }   }
    return(ex);
#   undef blin_internal_flags
}
